@@ -5,41 +5,62 @@
 #' @author Jaime Mosquera Gutiérrez, \email{jmosquerag@unal.edu.co}
 #'
 #' @description
+#' `r lifecycle::badge("experimental")`
+#'
 #' Function to compute maximum likelihood estimators (MLE) of regression parameters
 #' of any distribution implemented in \code{R} with covariates (linear predictors).
 #'
-#' @param formulas a list of formula objects. Each element must have an \code{~}, with the terms
-#' on the right separated by \code{+} operators. The response variable on the left side is optional.
-#' Linear predictor of each parameter must be specified with the name of the parameter followed by
-#' the suffix \code{'.fo'}. See the examples below for further illustration.
-#' @param y_dist a formula object that specifies the distribution of the response variable.
-#'               On the left side of \code{~} must be the response, and in the right side
-#'               must be the name o de probability density/mass function. See the section
-#'               \strong{Details} and the examples below for further illustration.
-#' @param data an optional data frame containing the variables in the model. If data is not
-#'             specified, the variables are taken from the environment from which
-#'             \code{maxlogLreg} is called.
+#' @param formulas a list of formula objects. Each element must have an \code{~},
+#' with the terms on the right separated by \code{+} operators. The response
+#' variable on the left side is optional. Linear predictor of each parameter must
+#' be specified with the name of the parameter followed by the suffix \code{'.fo'}.
+#' See the examples below for further illustration.
+#' @param y_dist a formula object that specifies the distribution of the response
+#'               variable. On the left side of \code{~} must be the response,
+#'               and in the right side must be the name o de probability
+#'               density/mass function. See the section \strong{Details} and the
+#'               examples below for further illustration.
+#' @param support a list with the following entries:
+#'                \itemize{
+#'                \item \code{interval}: a two dimensional atomic vector indicating
+#'                      the set of possible values of a random variable having the
+#'                      distribution specified in \code{y_dist}.
+#'                \item \code{type}: character indicating if distribution has a
+#'                \code{discrete} or a \code{continous} random variable.
+#'                }
+#' @param data an optional data frame containing the variables in the model.
+#'             If data is not specified, the variables are taken from the
+#'             environment from which \code{maxlogLreg} is called.
 #' @param subset an optional vector specifying a subset of observations to be used
 #'               in the fitting process.
-#' @param fixed a list with fixed/known parameters of distribution of interest. Fixed parameters
-#'              must be passed with its name and its value (known).
-#' @param link a list with names of parameters to be linked, and names of the link function object.
-#'             For names of parameters, please visit documentation of density/mass function.
-#'             There are three link functions available: \code{\link{log_link}},
-#'             \code{\link{logit_link}} and \code{\link{NegInv_link}}. Take into account: the order
-#'             used in argument \code{over} corresponds to the order in argument \code{link}.
-#' @param start a numeric vector with initial values for the parameters to be estimated. Zero is the
-#'              default value.
+#' @param fixed a list with fixed/known parameters of distribution of interest.
+#'              Fixed parameters must be passed with its name and its value (known).
+#' @param link a list with names of parameters to be linked, and names of the
+#'             link function object. For names of parameters, please visit
+#'             documentation of density/mass function. There are three link
+#'             functions available: \code{\link{log_link}}, \code{\link{logit_link}}
+#'             and \code{\link{NegInv_link}}. Take into account: the order
+#'             used in argument \code{over} corresponds to the order in argument
+#'             \code{link}.
+#' @param start a numeric vector with initial values for the parameters to be
+#'              estimated. Zero is the default value.
 #' @param lower a numeric vector with lower bounds, with the same lenght of
-#'              argument `start` (for box-constrained optimization). \code{-Inf} is the default value.
+#'              argument `start` (for box-constrained optimization). \code{-Inf}
+#'              is the default value.
 #' @param upper a numeric vector with upper bounds, with the same lenght of
-#'              argument `start` (for box-constrained optimization). \code{Inf} is the default value.
-#' @param optimizer a lenght-one character vector with the name of optimization routine.
-#'                  \code{\link{nlminb}}, \code{\link{optim}} and
+#'              argument `start` (for box-constrained optimization). \code{Inf}
+#'              is the default value.
+#' @param optimizer a length-one character vector with the name of optimization
+#'                  routine.  \code{\link{nlminb}}, \code{\link{optim}} and
 #'                  \code{\link[DEoptim]{DEoptim}} are available; \code{\link{nlminb}}
 #'                  is the default routine.
-#' @param control control parameters of the optimization routine. Please, visit documentation of selected
-#'                optimizer for further information.
+#' @param control control parameters of the optimization routine. Please, visit
+#'                documentation of selected optimizer for further information.
+#' @param StdE_method a length-one character vector with the routine for Hessian
+#'                    matrix computation. The This is needed for standard error
+#'                    estimation. The options available are \code{"optim"} and
+#'                    \code{"numDeriv"}. For further information, visit
+#'                    \code{\link[stats]{optim}} or \code{\link[numDeriv]{hessian}}.
 #' @param silent  logical. If TRUE, warnings of \code{maxlogL} are suppressed.
 #' @param ... Further arguments to be supplied to the optimization routine.
 #'
@@ -47,25 +68,65 @@
 #'  lists:
 #' \item{fit}{A list with output information about estimation and method used.}
 #' \item{inputs}{A list with all input arguments.}
-#' \item{outputs}{A list with additional information:
+#' \item{outputs}{A list with additional information. The most important outputs
+#'       are:
 #'       \itemize{
-#'          \item Number of parameters.
-#'          \item Sample size
-#'          \item Standard error computation method.
-#'          \item Number of regression parameters.
+#'          \item \code{npar}: number of parameters.
+#'          \item \code{n}: sample size
+#'          \item \code{Stde_method}: standard error computation method.
+#'          \item \code{b_lenght}: a list with the number of regression parameters.
+#'          \item \code{design_matrix}: a list with the \eqn{\mathbf{X}} matrix
+#'                for each parameter, the response values (called \code{y}) and
+#'                the censorship matrix (called \code{status}). See the Details
+#'                section for further information.
 #'        }
 #' }
 #'
-#' @details \code{maxlogLreg} calculates computationally the log-likelihood (log L) function
-#' corresponding to the distribution specified in argument \code{y_dist} with linear
-#' predictors specified in argument \code{formulas}. Then, it maximizes the log L through
-#' \code{\link{optim}}, \code{\link{nlminb}} or \code{\link{DEoptim}}. \code{maxlogLreg}
-#' generates an S3 object of class \code{maxlogL}.
+#' @details \code{maxlogLreg} computes programmatically the log-likelihood
+#' (log L) function corresponding for the following model:
 #'
-#' Noncentrality parameters must be named as \code{ncp} in the distribution.
+#' \deqn{
+#'   y_i \stackrel{iid.}{\sim} \mathcal{D}(\theta_{i1},\theta_{i2},\dots,
+#'   \theta_{ij}, \dots, \theta_{ik})
+#' }
+#' \deqn{
+#'   g(\boldsymbol{\theta}_{j}) = \boldsymbol{\eta}_{j} = \mathbf{X}_j^\top
+#'   \boldsymbol{\beta}_j,
+#' }
 #'
-#' @note The following generic functions can be used with a \code{maxlogL} object:
-#' \code{summary, print, logLik, AIC}.
+#' where,
+#'
+#' \itemize{
+#'   \item \eqn{g_k(\cdot)} is the \eqn{k}-th link function.
+#'   \item \eqn{\boldsymbol{\eta}_{j}} is the value of the linear predictor for the
+#'         $j^{th}$ for all the observations.
+#'   \item \eqn{\boldsymbol{\beta}_j = (\beta_{0j}, \beta_{1j},\dots,
+#'         \beta_{(p_j-1)j})^\top} are the fixed effects vector, where \eqn{p_j}
+#'         is the number of parameters in linear predictor \eqn{j} and
+#'         \eqn{\mathbf{X}_j} is a known design matrix of order \eqn{n\times p_j}.
+#'         These terms are specified in \code{formulas} argument.
+#'   \item \eqn{\mathcal{D}} is the distribution specified in argument
+#'         \code{y_dist}.
+#' }
+#'
+#' Then, \code{maxlogLreg} maximizes the log L through \code{\link{optim}},
+#' \code{\link{nlminb}} or \code{\link{DEoptim}}. \code{maxlogLreg} generates
+#' an S3 object of class \code{maxlogL}.
+#'
+#' Estimation with censorship can be handled with \code{Surv} objects
+#' (see example 2). The output object stores the corresponding censorship matrix,
+#' defined as \eqn{r_{il} = 1} if sample unit \eqn{i} has status \eqn{l}, or
+#' \eqn{r_{il} = 0} in other case. \eqn{i=1,2,\dots,n} and \eqn{l=1,2,3}
+#' (\eqn{l=1}: observation status, \eqn{l=2}: right censorship status,
+#' \eqn{l=3}: left censorship status).
+#'
+#' @note
+#' \itemize{
+#'   \item The following generic functions can be used with a \code{maxlogL}
+#'   object: \code{summary, print, logLik, AIC}.
+#'   \item Noncentrality parameters must be named as \code{ncp} in the
+#'   distribution.
+#' }
 #'
 #' @importFrom stats nlminb optim pnorm model.frame as.formula na.omit
 #' @importFrom survival Surv
@@ -86,8 +147,10 @@
 #'
 #' # It does not matter the order of distribution parameters
 #' formulas <- list(sd.fo = ~ x, mean.fo = ~ x)
+#' support <- list(interval = c(-Inf, Inf), type = 'continuous')
 #'
-#' norm_mod <- maxlogLreg(formulas, y_dist = y ~ dnorm, data = norm_data,
+#' norm_mod <- maxlogLreg(formulas, y_dist = y ~ dnorm, support = support,
+#'                        data = norm_data,
 #'                        link = list(over = "sd", fun = "log_link"))
 #' summary(norm_mod)
 #'
@@ -103,6 +166,7 @@
 #'
 #' # Formulas with linear predictors
 #' formulas <- list(scale.fo=~1, shape.fo=~1)
+#' support <- list(interval = c(0, Inf), type = 'continuous')
 #'
 #' # Bounds for optimization. Upper bound set with default values (Inf)
 #' start <- list(
@@ -115,7 +179,7 @@
 #' )
 #'
 #' mod_weibull <- maxlogLreg(formulas, y_dist = Surv(fails, status) ~ dweibull,
-#'                           start = start,
+#'                           support = c(0, Inf), start = start,
 #'                           lower = lower, data = Wei_data)
 #' summary(mod_weibull)
 #'
@@ -141,10 +205,11 @@
 # Maximization routine for regression -----------------------------------------
 #==============================================================================
 #' @export
-maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
-                       fixed = NULL, link = NULL, start = NULL,
+maxlogLreg <- function(formulas, y_dist, support = NULL, data = NULL,
+                       subset = NULL, fixed = NULL, link = NULL, start = NULL,
                        lower = NULL, upper = NULL, optimizer = 'nlminb',
-                       control = NULL, silent = FALSE, ...){
+                       control = NULL, silent = FALSE,
+                       StdE_method = c('optim', 'numDeriv'), ...){
 
   if (silent) options(warn = -1)
   call <- match.call()
@@ -255,6 +320,7 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
                                  y_dist = y_dist, npar = npar,
                                  par_names = par_names)
   levels <- dsgn_mat$levels
+  cens <- dsgn_mat$status
 
   ## Number of regression parameters
   n_betas <- sum(as.numeric(unlist(sapply(dsgn_mat[1:npar], ncol))))
@@ -319,7 +385,7 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
                    distr = distr, dist_args = arguments, over = link$over,
                    link = link$fun, npar = npar, fixed = fixed,
                    par_names = par_names, b_length = b_length)
-    fit$par <- fit$optim$bestmem
+    fit$par <- as.numeric(fit$optim$bestmem)
     fit$objective <- -fit$optim$bestval
   }
 
@@ -331,7 +397,9 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
   names(fit$par) <- regpar_names
 
   # Hessian computation
-  fit$hessian <- try(optim(par = fit$par, fn = minus_lL_LinReg,
+  StdE_method <- match.arg(StdE_method, c('optim', 'numDeriv'))
+  if ( StdE_method == 'optim' ){
+  	fit$hessian <- try(optim(par = fit$par, fn = minus_lL_LinReg,
                            method = 'L-BFGS-B',
                            lower = fit$par - 0.5*fit$par,
                            upper = fit$par + 0.5*fit$par,
@@ -340,10 +408,10 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
                            link = link$fun, npar = npar, fixed = fixed,
                            par_names = par_names, b_length = b_length)$hessian,
                      silent = TRUE)
-
-  StdE_Method <- "Hessian from optim"
+  	StdE_computation <- "Hessian from optim"
+  }
   if ( (any(is.na(fit$hessian)) | is.error(fit$hessian)) |
-       any(is.character(fit$hessian)) ){
+       any(is.character(fit$hessian)) | StdE_method == 'numDeriv' ){
     fit$hessian <- try(numDeriv::hessian(minus_lL_LinReg, fit$par,
                                          mat = dsgn_mat, distr = distr,
                                          dist_args = arguments,
@@ -351,11 +419,11 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
                                          npar = npar, fixed = fixed,
                                          par_names = par_names,
                                          b_length = b_length), silent = TRUE)
-    StdE_Method <- "numDeriv::hessian"
+    StdE_computation <- "numDeriv::hessian"
   }
   if ( (any(is.na(fit$hessian)) | is.error(fit$hessian)) |
        any(is.character(fit$hessian)) ){
-    StdE_Method <- "'optim' and 'numDeriv' failed"
+    StdE_computation <- paste0("'", StdE_method, "' failed")
     fit$hessian <- NA
     fit$StdE <- NA
   } else {
@@ -374,6 +442,7 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
                               dist_args = arguments, npar = npar,
                               link_fun = link$fun)
   names(fitted.values) <- par_names
+  fitted.values <- lapply(fitted.values, function(x) as.numeric(x))
 
   # Coefficients of predictors
   A <- param_index(b_length, npar)
@@ -382,12 +451,12 @@ maxlogLreg <- function(formulas, y_dist, data = NULL, subset = NULL,
 
   # fit stores the following information:
   # fit <- list(par, objective, hessian, StdE)
-  inputs <- list(call = call, distr = distr, y_dist = y_dist,
-                 formulas = formulas, fixed = fixed, link = link,
+  inputs <- list(call = call, distr = distr, y_dist = y_dist, support = support,
+                 formulas = formulas, fixed = fixed, link = link, cens = cens,
                  start = start, lower = lower, upper = upper,
                  optimizer = optimizer, data = dsgn_mat$data)
   outputs <- list(npar = npar - length(fixed), n = length(dsgn_mat$y),
-                  StdE_Method = StdE_Method, type = "maxlogLreg",
+                  StdE_Method = StdE_computation, type = "maxlogLreg",
                   b_length = b_length, levels = levels,
                   par_names = par_names, response = dsgn_mat$y,
                   design_matrix = dsgn_mat,
@@ -579,7 +648,7 @@ Surv_transform <- function(y_dist, data){
 # log-likelihood function computation -----------------------------------------
 #==============================================================================
 minus_lL_LinReg <- function(param, mat, distr, dist_args, over, link, npar,
-                            fixed, par_names, b_length){
+                            fixed, par_names, b_length, summation = TRUE){
     # Linear predictor
     betas_list <- all_betas(b_length = b_length, npar = npar,
                             param = param)
@@ -605,7 +674,7 @@ minus_lL_LinReg <- function(param, mat, distr, dist_args, over, link, npar,
     }
     y <- mat$y
     delta <- mat$status
-    cdf  <- paste0("p", substring(distr, 2))
+    cdf  <- paste0('p', substring(distr, 2))
 
     logf <- do.call( what = distr, args = c(list(x = y), param,
                                            log = TRUE, fixed) )
@@ -618,7 +687,7 @@ minus_lL_LinReg <- function(param, mat, distr, dist_args, over, link, npar,
     ll <- sum( logf*delta[,1] + logF*delta[,2] + logS*delta[,3] )
 
     # Negative of log-Likelihood function
-    return(as.numeric(-ll)) # Useful when using Rmpfr (momi)
+    return(as.numeric(-ll)) # Useful when using Rmpfr (LuA)
 }
 param_index <- function(b_length, npar){
   b_length_plus <- c(0, as.numeric(b_length))
